/* 
  Bond.h

  Barry McMullin <mcmullin@eeng.dcu.ie>
  01-OCT-1996


*/


#import <objectbase.h>
#import <simtools.h>
#import <tkobjc/Raster.h>

#import "OrientationCode.h"
#import "Agent.h"

@class Link;

@interface Bond : Agent {
  id link0, link1;
  NeighborCode neighborCode0, neighborCode1;
  OrientationCode orientationCode;
}

/* This Bernoulli distribution is used in implementing
   the random decay of bonds.  It must be a "variable"
   or non-"frozen" flavour of Bernoulli, created
   externally to the bonds; in this way a single
   bernoulli object can be shared around multiple
   usages in a single app.  */
+(void) setBernoulli: (id) inBernoulli; 
+(id) getBernoulli;

+(void) setBondDecayProbability: (double) probability;
+(double) getBondDecayProbability;

-setLink0: (Link *) link;
-(Link *) getLink0;
-setNeighbor0: (NeighborCode) neighbor;
-(NeighborCode) getNeighbor0;


-setLink1: (Link *) link;
-(Link *) getLink1;
-setNeighbor1: (NeighborCode) neighbor;
-(NeighborCode) getNeighbor1;

-setOrientationCode: (OrientationCode) inOrientationCode;
-(OrientationCode) getOrientationCode;

-(Link *) getLinkOppositeTo: (Link *) link;

-(NeighborCode) getNeighborCodeForLink: (Link *) link;

-clearLinks;

-step;

-(void) drop;

-saveTo: (id <OutFile>) file;   // Overridden...
-loadFrom: (id <InFile>) file toLattice: (id) inLattice;  // Overridden...

-(void) xprintStatus;  // Debug use only...



@end
