/* 
  Bond.m

  Barry McMullin <mcmullin@eeng.dcu.ie>
  01-OCT-1996


*/


#import "SCLGlobals.h"
#import "Bernoulli.h"
#import "Lattice.h"
#import "AgentManager.h"
#import "Link.h"
#import "Bond.h"


@implementation Bond

static id bernoulli;
static double bondDecayProbability = 0.1;
/* Pseudo Class variables. */

+(void) setBernoulli: (id) inBernoulli {
  bernoulli = inBernoulli;
}

+(id) getBernoulli {
  return(bernoulli);
}


+(void) setBondDecayProbability: (double) probability {
  bondDecayProbability = probability;
}

+(double) getBondDecayProbability {
  return bondDecayProbability;
}

-setLink0: (Link *) link {
  link0 = link;

  return self;
}

-(Link *) getLink0 {
  return link0;
}

-setNeighbor0: (NeighborCode) neighborCode {
  neighborCode0 = neighborCode;

  return self;
}

-(NeighborCode) getNeighbor0 {
  return neighborCode0;
}

-setLink1: (Link *) link {
  link1 = link;

  return self;
}

-(Link *) getLink1 {
  return link1;
}

-setNeighbor1: (NeighborCode) neighborCode {
  neighborCode1 = neighborCode;

  return self;
}

-(NeighborCode) getNeighbor1 {
  return neighborCode1;
}


-setOrientationCode: (OrientationCode) inOrientationCode {
  orientationCode = inOrientationCode;

  return self;
}

-(OrientationCode) getOrientationCode {
  return orientationCode;
}

-(Link *) getLinkOppositeTo: (Link *) link {
  if (link == link0)
    return (link1);
  else {
    if (link == link1)
      return (link0);
    else {
      [InternalError raiseEvent:
        "Inconsistent call to getLinkOppositeTo: on bond %p,\n"
        "from link: %p\n",
        self, link];
      return (nil);
    }
  }
}

-(NeighborCode) getNeighborCodeForLink: (Link *) link {
  if (link == link0)
    return (neighborCode0);
  else {
    if (link == link1)
      return (neighborCode1);
    else {
      [InternalError raiseEvent:
        "Inconsistent call to getNeighborCodeForLink: on bond %p,\n"
        "from link: %p\n",
        self, link];
      return (Center); // Defensive...
    }
  }


}

-clearLinks {
  if ((link0 == nil) || (link1 == nil))
    [InternalError raiseEvent:
       "Nil link encountered on bond %p.\n"
       "link0: %p  link1: %p\n",
       self, link0, link1];

  [link0 removeBond: self];
  link0 = nil;
  neighborCode0 = Center;
  [link1 removeBond: self];
  link1 = nil;
  neighborCode1 = Center;

  return self;
}



-(void) drop {
  if ((link0 != nil) || (link1 != nil))
    [self clearLinks];

  [super drop];
}

-step {
  if ([bernoulli getTossWithProb: bondDecayProbability]) {
    [self clearLinks];
    [bondMgr dropAgent: self];
  }

  return self;
}

-saveTo: (id <OutFile>) file {
  id coord0, coord1;

  if ((link0 == nil) || (link1 == nil))
    [InternalError raiseEvent:
       "Attempt to save bond %d with nil link(s).\n"
       "link0: %d  link1: %d\n",
       self, link0, link1];

  [super saveTo: file];

  coord0 = [link0 copyCoord: [self getZone]];
  [coord0 saveTo: file];
  [coord0 drop];

  [file putInt: neighborCode0];
  [file putString: " "];

  coord1 = [link1 copyCoord: [self getZone]];
  [coord1 saveTo: file];
  [coord1 drop];

  [file putInt: neighborCode1];
  [file putString: " "];

  [file putInt: orientationCode];
  [file putString: " "];

  return self;
}

-loadFrom: (id <InFile>) file toLattice: (id) inLattice {
  id coord0, coord1;
  id bondAgentIndex;
  int neighborInt;
  int orientationInt;
  int myOffset;

  [super loadFrom: file];

  if ((link0 != nil) || (link1 != nil))
    [InternalError raiseEvent:
       "Attempt to load bond %d with non-nil link(s).\n"
       "link0: %d  link1: %d\n",
       self, link0, link1];

  bondAgentIndex = [[bondMgr getAgentList] begin: [self getZone]];
  if (![bondAgentIndex findNext: self]) 
    [InternalError raiseEvent:
      "bond %p  missing from bondAgentList.\n",
      self];
  myOffset = [bondAgentIndex getOffset];
  [bondAgentIndex drop];

  coord0 = [inLattice createOriginCoord: [self getZone]];
  [coord0 loadFrom: file];

  [file getInt: &neighborInt];
  neighborCode0 = (NeighborCode) neighborInt;
  if (![inLattice isValidNeighborCode: neighborCode0])
    [InternalError raiseEvent:
      "Invalid neighbor code %d loaded.\n", neighborCode0];

  link0 = [coord0 getObject];
  if (![link0 isMemberOf: [Link class]])
    [InternalError raiseEvent:
       "Attempt to bond non-link particle %d.\n", link0];
  [link0 restoreBond: self Offset: myOffset];
  [coord0 drop];


  coord1 = [inLattice createOriginCoord: [self getZone]];
  [coord1 loadFrom: file];

  [file getInt: &neighborInt];
  neighborCode1 = (NeighborCode) neighborInt;
  if (![inLattice isValidNeighborCode: neighborCode1])
    [InternalError raiseEvent:
      "Invalid neighborCode code %d loaded.\n", neighborCode1];

  link1 = [coord1 getObject];
  if (![link1 isMemberOf: [Link class]])
    [InternalError raiseEvent:
       "Attempt to bond non-link particle %d.\n", link1];
  [link1 restoreBond: self Offset: myOffset];
  [coord1 drop];

  [file getInt: &orientationInt];
  orientationCode = (OrientationCode) orientationInt;
  switch (orientationCode) {
    case SqrNorthSouth:
    case SqrEastWest:
    case SqrNorthEastSouthWest:
    case SqrNorthWestSouthEast:
    case TriNorthSouth:
    case TriNorthWestSouthEast:
    case TriNorthEastSouthWest:
      break;
    default:
      [InternalError raiseEvent: 
	"Invalid orientation code %d loaded.\n", orientationCode];
  }

  return self;
}

-(void) xprintStatus {
  printf("Bond: xprintStatus (self: %p)\n", self);
  printf("  [super xprintStatus]:\n");
  [super xprintStatus];
  printf("  link0: %p  link1: %p\n", link0, link1);
  printf("  neighborCode0: %d  neighborCode1: %d\n", 
    neighborCode0, neighborCode1);
  printf("  orientationCode: %d\n", orientationCode);
  fflush(stdout);
}


@end
