/* 
  CellEditor.m

  Barry McMullin <mcmullin@eeng.dcu.ie>
  07-OCT-1996

*/

#import <simtools.h>

#import "SCLGlobals.h"
#import "SCLModelSwarm.h"
#import "Hole.h"
#import "Substrate.h"
#import "Link.h"
#import "Catalyst.h"
#import "ParticleManager.h"
#import "Lattice2d.h"

#import "CellEditor.h"

@implementation CellEditor

+create: (id) aZone withLattice: (id) inLattice {
  CellEditor *finalSelf;

  finalSelf = [super createBegin: aZone];
  finalSelf->coord = [inLattice createOriginCoord: [finalSelf getZone]];
  finalSelf = [finalSelf createEnd];

  return finalSelf;
}


-update {
  agent = [coord getObject];
  if (agent == nil)
    agentClass = nil;
  else
    agentClass = [agent class];

  return self;
}

-setRasterX: (int) newX Y: (int) newY {
  [coord setRasterX: newX];
  [coord setRasterY: newY];

  [self update];

  return self;
}

-setWorldX: (int) newX Y: (int) newY {
  [coord setX: newX];
  [coord setY: newY];

  [self update];

  return self;
}

-forceCatalyst {
  Catalyst *catalyst;

  if (agentClass != [Catalyst class]) {
    [worldManager dropAgent: agent];
    catalyst = [catalystMgr createAgent];
    if (![catalyst attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp catalyst.\n"];
    [self update];
    [worldManager update];
  }

  return self;
}
    
-forceLink {
  Link *link;

  if (agentClass != [Link class]) {
    [worldManager dropAgent: agent];
    link = [linkMgr createAgent];
    if (![link attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp link.\n"];
    [self update];
    [worldManager update];
  }

  return self;
}

-forceSubstrate {
  Substrate *substrate;

  if (agentClass != [Substrate class]) {
    [worldManager dropAgent: agent];
    substrate = [substrateMgr createAgent];
    if (![substrate attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp substrate.\n"];
    [self update];
    [worldManager update];
  }

  return self;
}
 

-forceHole {
  Hole *hole;

  if (agentClass != [Hole class]) {
    [worldManager dropAgent: agent];
    hole = [holeMgr createAgent];
    if (![hole attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp hole.\n"];
    [self update];
    [worldManager update];
  }

  return self;
}
     

@end
