/* 
  Lattice.h

  This defines the <Lattice> protocol, which is an
  attempt at a generalised protocol suitable for "any"
  kind of "regular", discrete lattice.  Conceptually,
  most of the functionality of such a Lattice will
  actually be offered via soime kind of tailored
  "Coord" objects, which identify "positions" in the
  Lattice. (See Coord.h) The Lattice object itself is
  mainly confined to providing Coord objects, through
  which the interaction can then be carried out.
  
  Barry McMullin <mcmullin@eeng.dcu.ie>
  FEB-1997

*/

#import <objectbase.h>

#import "Neighborhood.h"

@protocol Lattice <Neighborhood>

-(id) createOriginCoord: (id) aZone;
-(id) createMaxCoord: (id) aZone;


/* If you wish to use any of the randomizing methods of
   the Neighborhood protocol, or the -createRandomCoord
   method defined here, a uniformUnsigned distribution
   object (of the "variable" or non-"frozen" kind) must
   be available to the Lattice object.  This must be
   provided from outside via -setUniformUnsigned
   (declared in the <Neighborhood> protocol). This
   approach is preferred to having the Lattice
   object create its own private distribution object,
   because it will be quite common to create a
   uniformUnsigned distribution object for more general
   use in an app anyway, and share it around whoever
   else wants to use it...  */

-(id) createRandomCoord: (id) aZone;
/* Yields a coord picked randomly, with a uniform
   distribution over all legal coords for the
   lattice. */

@end
