/* 
  ParameterManager.m

  Barry McMullin <mcmullin@eeng.dcu.ie>
  16-OCT-1996

*/

#import <simtools.h>
#import <simtoolsgui.h>

#import "SCLGlobals.h"
#import "Version.h"

#import "Hole.h"
#import "Substrate.h"
#import "Link.h"
#import "Catalyst.h"
#import "Bond.h"

#import "ParameterManager.h"

@implementation ParameterManager


-createEnd {
  ParameterManager *finalSelf;
  id <MessageProbe> messageProbe;
  id <ProbeMap> probeMap;

  finalSelf = [super createEnd];

  [finalSelf setCatalystMobilityFactor: 0.05];
  [finalSelf setProductionProbability: 1.0];
  [finalSelf setCatalystDecayProbability: 0.1];
  [finalSelf setLink0MobilityFactor: 0.1];
  [finalSelf setLink1MobilityFactor: 0.1];
  [finalSelf setLink2MobilityFactor: 0.1];
  [finalSelf setDisintegrationProbability: 0.01];
  [finalSelf setNoBondingProbability: 0.25];
  [finalSelf setChainInitiateProbability: 0.25];
  [finalSelf setChainExtendProbability: 0.25];
  [finalSelf setChainSpliceProbability: 0.25];
  [finalSelf setSubstrateMobilityFactor: 0.5];
  [finalSelf setAbsorptionProbability: 0.5];
  [finalSelf setEmissionProbability: 0.5];
  [finalSelf setActiveRepairProbability: 0.5];
  [finalSelf setDisplaceProbability: 0.01];
  [finalSelf setHoleMobilityFactor: 0.5];
  [finalSelf setBondDecayProbability: 0.0];
  [finalSelf setLinkPromotionProbability: 0.1];
  [finalSelf setChainInhibitBondFlag: YES];
  [finalSelf setCatInhibitBondFlag: NO];
  [finalSelf setCatInhibitDisintFlag: NO];
  [finalSelf setEnergeticMotionFlag: NO];
  [finalSelf setAcuteBondingFlag: NO];
  [finalSelf setCatSmartMotionFlag: NO];
  [finalSelf setLink0SmartMotionFlag: NO];
  [finalSelf setLink1SmartMotionFlag: NO];

  probeMap = [EmptyProbeMap createBegin: [finalSelf getZone]];
  [probeMap setProbedClass: [finalSelf class]];
  probeMap = [probeMap createEnd];

  messageProbe = 
    [probeLibrary getProbeForMessage: "saveToFileNamed:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "loadFromFileNamed:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "catalystMobilityFactor"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "productionProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "catalystDecayProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "link0MobilityFactor"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "link1MobilityFactor"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "link2MobilityFactor"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "disintegrationProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "noBondingProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "chainInitiateProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "chainExtendProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "chainSpliceProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "absorptionProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "emissionProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "activeRepairProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "displaceProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "substrateMobilityFactor"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "holeMobilityFactor"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "bondDecayProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "linkPromotionProbability"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "chainInhibitBondFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "catInhibitBondFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "catInhibitDisintFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "energeticMotionFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "acuteBondingFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "catSmartMotionFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "link0SmartMotionFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "link1SmartMotionFlag"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "timerKludge"
       inClass: [finalSelf class]]];

  [probeLibrary setProbeMap: probeMap For: [finalSelf class]];

  finalSelf->probeDisplay =
    [probeDisplayManager createProbeDisplayFor: finalSelf];


  return (finalSelf);
}

-(void) drop {
  [probeDisplay drop];
  [super drop];
}


-(void) setCatalystMobilityFactor: (double) mobilityFactor {
  catalystMobilityFactor = mobilityFactor;
  [Catalyst setClassMobilityFactor: catalystMobilityFactor];
}

-(double) getCatalystMobilityFactor {
  catalystMobilityFactor  = [Catalyst getClassMobilityFactor]; // Defensive...

  return catalystMobilityFactor;
}

-(void) setProductionProbability: (double) probability {
  productionProbability = probability;
  [Catalyst setProductionProbability: productionProbability];
}

-(double) getProductionProbability {
  productionProbability = [Catalyst getProductionProbability]; // Defensive...

  return productionProbability;
}

-(void) setCatalystDecayProbability: (double) probability {
  catalystDecayProbability = probability;
  [Catalyst setCatalystDecayProbability: catalystDecayProbability];
}

-(double) getCatalystDecayProbability {
  catalystDecayProbability = [Catalyst getCatalystDecayProbability]; 
    // Defensive...

  return catalystDecayProbability;
}

-(void) setLink0MobilityFactor: (double) mobilityFactor {
  link0MobilityFactor = mobilityFactor;
  [Link setLink0MobilityFactor: link0MobilityFactor];
}

-(double) getLink0MobilityFactor {
  link0MobilityFactor = [Link getLink0MobilityFactor]; // Defensive...

  return link0MobilityFactor;
}

-(void) setLink1MobilityFactor: (double) mobilityFactor {
  link1MobilityFactor = mobilityFactor;
  [Link setLink1MobilityFactor: link1MobilityFactor];
}

-(double) getLink1MobilityFactor {
  link1MobilityFactor = [Link getLink1MobilityFactor]; // Defensive...

  return link1MobilityFactor;
}

-(void) setLink2MobilityFactor: (double) mobilityFactor {
  link2MobilityFactor = mobilityFactor;
  [Link setLink2MobilityFactor: link2MobilityFactor];
}

-(double) getLink2MobilityFactor {
  link2MobilityFactor = [Link getLink2MobilityFactor]; // Defensive...

  return link2MobilityFactor;
}

-(void) setLinkPromotionProbability: (double) probability {
  linkPromotionProbability = probability;
  [Link setLinkPromotionProbability: linkPromotionProbability];
}

-(double) getLinkPromotionProbability {
  linkPromotionProbability = [Link getLinkPromotionProbability]; 
    // Defensive...

  return linkPromotionProbability;
}

-(void) setChainInhibitBondFlag: (BOOL) value {
  chainInhibitBondFlag = value;
  [Link setChainInhibitBondFlag: chainInhibitBondFlag]; 
}

-(BOOL) getChainInhibitBondFlag {
  chainInhibitBondFlag = 
    [Link getChainInhibitBondFlag]; // Defensive...

  return chainInhibitBondFlag;
}

-(void) setCatInhibitBondFlag: (BOOL) value {
  catInhibitBondFlag = value;
  [Link setCatInhibitBondFlag: catInhibitBondFlag]; 
}

-(BOOL) getCatInhibitBondFlag {
  catInhibitBondFlag = 
    [Link getCatInhibitBondFlag]; // Defensive...

  return catInhibitBondFlag;
}

-(void) setCatInhibitDisintFlag: (BOOL) value {
  catInhibitDisintFlag = value;
  [Link setCatInhibitDisintFlag: catInhibitDisintFlag]; 
}

-(BOOL) getCatInhibitDisintFlag {
  catInhibitDisintFlag = 
    [Link getCatInhibitDisintFlag]; // Defensive...

  return catInhibitDisintFlag;
}

-(void) setEnergeticMotionFlag: (BOOL) value {
  energeticMotionFlag = value;
  [Particle setEnergeticMotionFlag: energeticMotionFlag]; 
}

-(BOOL) getEnergeticMotionFlag {
  energeticMotionFlag = 
    [Particle getEnergeticMotionFlag]; // Defensive...

  return energeticMotionFlag;
}

-(void) setAcuteBondingFlag: (BOOL) value {
  acuteBondingFlag = value;
  [Link setAcuteBondingFlag: acuteBondingFlag]; 
}

-(BOOL) getAcuteBondingFlag {
  acuteBondingFlag = 
    [Link getAcuteBondingFlag]; // Defensive...

  return acuteBondingFlag;
}

-(void) setCatSmartMotionFlag: (BOOL) value {
  catSmartMotionFlag = value;
  [Catalyst setCatSmartMotionFlag: catSmartMotionFlag]; 
}

-(BOOL) getCatSmartMotionFlag {
  catSmartMotionFlag = 
    [Catalyst getCatSmartMotionFlag]; // Defensive...

  return catSmartMotionFlag;
}

-(void) setLink0SmartMotionFlag: (BOOL) value {
  link0SmartMotionFlag = value;
  [Link setLink0SmartMotionFlag: link0SmartMotionFlag]; 
}

-(BOOL) getLink0SmartMotionFlag {
  link0SmartMotionFlag = 
    [Link getLink0SmartMotionFlag]; // Defensive...

  return link0SmartMotionFlag;
}

-(void) setLink1SmartMotionFlag: (BOOL) value {
  link1SmartMotionFlag = value;
  [Link setLink1SmartMotionFlag: link1SmartMotionFlag]; 
}

-(BOOL) getLink1SmartMotionFlag {
  link1SmartMotionFlag = 
    [Link getLink1SmartMotionFlag]; // Defensive...

  return link1SmartMotionFlag;
}

-(void) setDisintegrationProbability: (double) probability {
  disintegrationProbability = probability;
  [Link setDisintegrationProbability: disintegrationProbability]; 
}

-(double) getDisintegrationProbability {
  disintegrationProbability = 
    [Link getDisintegrationProbability]; // Defensive...

  return disintegrationProbability;
}

-(void)setNoBondingProbability: (double) probability {
  noBondingProbability = probability;
  [Link setNoBondingProbability: noBondingProbability];
}

-(double) getNoBondingProbability {
  noBondingProbability = [Link getNoBondingProbability]; // Defensive...

  return noBondingProbability;
}

-(void)setChainInitiateProbability: (double) probability {
  chainInitiateProbability = probability;
  [Link setChainInitiateProbability: chainInitiateProbability];
}

-(double) getChainInitiateProbability {
  chainInitiateProbability = [Link getChainInitiateProbability]; // Defensive...

  return chainInitiateProbability;
}

-(void) setChainExtendProbability: (double) probability {
  chainExtendProbability = probability;
  [Link setChainExtendProbability: chainExtendProbability];
}

-(double) getChainExtendProbability {
  chainExtendProbability = [Link getChainExtendProbability]; // Defensive...

  return chainExtendProbability;
}

-(void) setChainSpliceProbability: (double) probability {
  chainSpliceProbability = probability;
  [Link setChainSpliceProbability: chainSpliceProbability];
}

-(double) getChainSpliceProbability {
  chainSpliceProbability = [Link getChainSpliceProbability]; // Defensive...

  return chainSpliceProbability;
}

-(void) setAbsorptionProbability: (double) probability {
  absorptionProbability = probability;
  [Link setAbsorptionProbability: absorptionProbability];
}

-(double) getAbsorptionProbability {
  absorptionProbability = [Link getAbsorptionProbability]; // Defensive...

  return absorptionProbability;
}

-(void) setEmissionProbability: (double) probability {
  emissionProbability = probability;
  [Link setEmissionProbability: emissionProbability];
}

-(double) getEmissionProbability {
  emissionProbability = [Link getEmissionProbability]; // Defensive...

  return emissionProbability;
}

-(void) setActiveRepairProbability: (double) probability {
  activeRepairProbability = probability;
  [Link setActiveRepairProbability: activeRepairProbability];
}

-(double) getActiveRepairProbability {
  activeRepairProbability = [Link getActiveRepairProbability]; // Defensive...

  return activeRepairProbability;
}

-(void) setDisplaceProbability: (double) probability {
  displaceProbability = probability;
  [Link setDisplaceProbability: displaceProbability];
}

-(double) getDisplaceProbability {
  displaceProbability = [Link getDisplaceProbability]; // Defensive...

  return displaceProbability;
}

-(void) setSubstrateMobilityFactor: (double) mobilityFactor {
  substrateMobilityFactor = mobilityFactor;
  [Substrate setClassMobilityFactor: substrateMobilityFactor];
}

-(double) getSubstrateMobilityFactor {
  substrateMobilityFactor = 
    [Substrate getClassMobilityFactor]; // Defensive...

  return substrateMobilityFactor;
}

-(void) setHoleMobilityFactor: (double) mobilityFactor {
  holeMobilityFactor = mobilityFactor;
  [Hole setClassMobilityFactor: holeMobilityFactor];
}

-(double) getHoleMobilityFactor {
  holeMobilityFactor = [Hole getClassMobilityFactor]; // Defensive...

  return holeMobilityFactor;
}

-(void) setBondDecayProbability: (double) probability {
  bondDecayProbability = probability;
  [Bond setBondDecayProbability: bondDecayProbability];
}

-(double) getBondDecayProbability {
  bondDecayProbability = [Bond getBondDecayProbability]; // Defensive...

  return bondDecayProbability;
}

-(void) setTimerKludge: (unsigned) kludge {
  timerKludge = kludge;
}

-(unsigned) getTimerKludge {
  return timerKludge;
}

-(void) apply {
  [Catalyst setClassMobilityFactor: catalystMobilityFactor];
  [Catalyst setProductionProbability: productionProbability];
  [Catalyst setCatalystDecayProbability: catalystDecayProbability];
  [Link setLink0MobilityFactor: link0MobilityFactor];
  [Link setLink1MobilityFactor: link1MobilityFactor];
  [Link setLink2MobilityFactor: link2MobilityFactor];
  [Link setLinkPromotionProbability: linkPromotionProbability];
  [Link setDisintegrationProbability: disintegrationProbability]; 
  [Link setNoBondingProbability: noBondingProbability];
  [Link setChainInitiateProbability: chainInitiateProbability];
  [Link setChainExtendProbability: chainExtendProbability];
  [Link setChainSpliceProbability: chainSpliceProbability];
  [Link setChainInhibitBondFlag: chainInhibitBondFlag];
  [Link setCatInhibitBondFlag: catInhibitBondFlag];
  [Link setCatInhibitDisintFlag: catInhibitDisintFlag];
  [Link setEnergeticMotionFlag: energeticMotionFlag];
  [Link setAcuteBondingFlag: acuteBondingFlag];
  [Catalyst setCatSmartMotionFlag: catSmartMotionFlag];
  [Link setLink0SmartMotionFlag: link0SmartMotionFlag];
  [Link setLink1SmartMotionFlag: link1SmartMotionFlag];
  [Link setAbsorptionProbability: absorptionProbability];
  [Link setEmissionProbability: emissionProbability];
  [Link setActiveRepairProbability: activeRepairProbability];
  [Link setDisplaceProbability: displaceProbability];
  [Substrate setClassMobilityFactor: substrateMobilityFactor];
  [Hole setClassMobilityFactor: holeMobilityFactor];
  [Bond setBondDecayProbability: bondDecayProbability];
}

-(void) slowdownKludge {
  unsigned i, j;

  [self apply];

  for (i = 0; i < timerKludge; i++) 
    for (j = 0; j < 100000; j++) ;
}

-saveTo: (id <OutFile>) outFile {
  [self apply]; // Just to be sure!

  [outFile putString: "Parameters...\n"];

  [outFile putString: "holeMobilityFactor:\n"];
  [outFile putDouble: [self getHoleMobilityFactor]];
  [outFile putNewLine];

  [outFile putString: "substrateMobilityFactor:\n"];
  [outFile putDouble: [self getSubstrateMobilityFactor]];
  [outFile putNewLine];

  [outFile putString: "link0MobilityFactor:\n"];
  [outFile putDouble: [self getLink0MobilityFactor]];
  [outFile putNewLine];
  [outFile putString: "link1MobilityFactor:\n"];
  [outFile putDouble: [self getLink1MobilityFactor]];
  [outFile putNewLine];
  [outFile putString: "link2MobilityFactor:\n"];
  [outFile putDouble: [self getLink2MobilityFactor]];
  [outFile putNewLine];
  [outFile putString: "noBondingProbability:\n"];
  [outFile putDouble: [self getNoBondingProbability]];
  [outFile putNewLine];
  [outFile putString: "chainInitiateProbability:\n"];
  [outFile putDouble: [self getChainInitiateProbability]];
  [outFile putNewLine];
  [outFile putString: "chainExtendProbability:\n"];
  [outFile putDouble: [self getChainExtendProbability]];
  [outFile putNewLine];
  [outFile putString: "chainSpliceProbability:\n"];
  [outFile putDouble: [self getChainSpliceProbability]];
  [outFile putNewLine];
  [outFile putString: "chainInhibitBondFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getChainInhibitBondFlag]];
  [outFile putNewLine];
  [outFile putString: "catInhibitBondFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getCatInhibitBondFlag]];
  [outFile putNewLine];
  [outFile putString: "catInhibitDisintFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getCatInhibitDisintFlag]];
  [outFile putNewLine];
  [outFile putString: "energeticMotionFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getEnergeticMotionFlag]];
  [outFile putNewLine];
  [outFile putString: "acuteBondingFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getAcuteBondingFlag]];
  [outFile putNewLine];
  [outFile putString: "catSmartMotionFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getCatSmartMotionFlag]];
  [outFile putNewLine];
  [outFile putString: "link0SmartMotionFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getLink0SmartMotionFlag]];
  [outFile putNewLine];
  [outFile putString: "link1SmartMotionFlag (NO = 0, YES = 1):\n"];
  [outFile putInt: (int) [self getLink1SmartMotionFlag]];
  [outFile putNewLine];
  [outFile putString: "disintegrationProbability:\n"];
  [outFile putDouble: [self getDisintegrationProbability]];
  [outFile putNewLine];
  [outFile putString: "absorptionProbability:\n"];
  [outFile putDouble: [self getAbsorptionProbability]];
  [outFile putNewLine];
  [outFile putString: "emissionProbability:\n"];
  [outFile putDouble: [self getEmissionProbability]];
  [outFile putNewLine];
  [outFile putString: "activeRepairProbability:\n"];
  [outFile putDouble: [self getActiveRepairProbability]];
  [outFile putNewLine];
  [outFile putString: "displaceProbability:\n"];
  [outFile putDouble: [self getDisplaceProbability]];
  [outFile putNewLine];
  [outFile putString: "linkPromotionProbability:\n"];
  [outFile putDouble: [self getLinkPromotionProbability]];
  [outFile putNewLine];

  [outFile putString: "catalystMobilityFactor:\n"];
  [outFile putDouble: [self getCatalystMobilityFactor]];
  [outFile putNewLine];
  [outFile putString: "productionProbability:\n"];
  [outFile putDouble: [self getProductionProbability]];
  [outFile putNewLine];

  [outFile putString: "catalystDecayProbability:\n"];
  [outFile putDouble: [self getCatalystDecayProbability]];
  [outFile putNewLine];

  [outFile putString: "bondDecayProbability:\n"];
  [outFile putDouble: [self getBondDecayProbability]];
  [outFile putNewLine];

  [outFile putString: "timerKludge:\n"];
  [outFile putInt: [self getTimerKludge]];
  [outFile putNewLine];

  return self;
}

-loadFrom: (id <InFile>) inFile {
  int intParm;
  double doubleParm;

  [inFile skipLine]; // "Parameters...\n"

  [inFile skipLine]; // "holeMobilityFactor:\n"
  [inFile getDouble: &doubleParm];
  [self setHoleMobilityFactor: doubleParm];
  [inFile skipLine];

  [inFile skipLine]; // "substrateMobilityFactor:\n"
  [inFile getDouble: &doubleParm];
  [self setSubstrateMobilityFactor: doubleParm];
  [inFile skipLine];

  [inFile skipLine]; // "link0MobilityFactor:\n"
  [inFile getDouble: &doubleParm];
  [self setLink0MobilityFactor: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "link1MobilityFactor:\n"
  [inFile getDouble: &doubleParm];
  [self setLink1MobilityFactor: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "link2MobilityFactor:\n"
  [inFile getDouble: &doubleParm];
  [self setLink2MobilityFactor: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "noBondingProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setNoBondingProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "chainInitiateProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setChainInitiateProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "chainExtendProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setChainExtendProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "chainSpliceProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setChainSpliceProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "chainInhibitBondFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setChainInhibitBondFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "catInhibitBondFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setCatInhibitBondFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "catInhibitDisintFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setCatInhibitDisintFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "energeticMotionFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setEnergeticMotionFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "acuteBondingFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setAcuteBondingFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "catSmartMotionFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setCatSmartMotionFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "link0SmartMotionFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setLink0SmartMotionFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "link1SmartMotionFlag (NO = 0, YES = 1):\n"
  [inFile getInt: &intParm];
  [self setLink1SmartMotionFlag: (BOOL) intParm];
  [inFile skipLine];
  [inFile skipLine]; // "disintegrationProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setDisintegrationProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "absorptionProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setAbsorptionProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "emissionProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setEmissionProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "activeRepairProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setActiveRepairProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "displaceProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setDisplaceProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "linkPromotionProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setLinkPromotionProbability: doubleParm];
  [inFile skipLine];

  [inFile skipLine]; //"catalystMobilityFactor:\n"
  [inFile getDouble: &doubleParm];
  [self setCatalystMobilityFactor: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "productionProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setProductionProbability: doubleParm];
  [inFile skipLine];
  [inFile skipLine]; // "catalystDecayProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setCatalystDecayProbability: doubleParm];
  [inFile skipLine];

  [inFile skipLine]; // "bondDecayProbability:\n"
  [inFile getDouble: &doubleParm];
  [self setBondDecayProbability: doubleParm];
  [inFile skipLine];

  [inFile skipLine]; // "timerKludge:\n"
  [inFile getInt: &intParm];
  [self setTimerKludge: intParm];
  [inFile skipLine];

  return self;
}

-saveToFileNamed: (char *) filename {
  id <OutFile> outFile;

  [modelSwarm setStateStopped]; // Defensive...

  outFile = [OutFile create: [self getZone] setName: filename];

  if (outFile == nil)
    [WarningMessage raiseEvent: 
    "parameterManager: saveToFileNamed: outFile is nil!\n"];
  else {
    [outFile putString: "# "];
    [outFile putString: [Version getAppIDString]];
    [outFile putNewLine];

    [self saveTo: outFile];

    [outFile drop];
  }

  return self;
}

-loadFromFileNamed: (char *) filename {
  id <InFile> inFile;

  [modelSwarm setStateStopped]; // Defensive...

  inFile = [InFile create: [self getZone] setName: filename];

  if (inFile == nil)
    [WarningMessage raiseEvent: 
    "parameterManager: loadFromFileNamed: inFile is nil!\n"];
  else {
    [inFile skipLine]; // Version info...
      // Possibly should check for version compatibility (;-)
    [self loadFrom: inFile];
    [inFile drop];
  }

  return self;
}

@end
