/* 
  PickNeighborCode.m

  Barry McMullin <mcmullin@eeng.dcu.ie>
  MAR-1997 
*/

#import <simtools.h>
#import <simtoolsgui.h>

#import "SCLGlobals.h"
#import "PickNeighborCode.h"
#import "Lattice.h"

#import "PickNeighborCode.h"

@implementation PickNeighborCode

+create: (id) aZone withLattice: (id) inLattice {
  PickNeighborCode *pickNeighborCode;

  pickNeighborCode = [super createBegin: aZone];
  pickNeighborCode->lattice = inLattice;
  pickNeighborCode = [pickNeighborCode createEnd];

  return (pickNeighborCode);
}

-createEnd {
  PickNeighborCode *finalSelf ;
  id <ProbeMap> probeMap;
  id <MessageProbe> messageProbe;

  finalSelf = [super createEnd];

  finalSelf->neighborCode = [lattice getInitialNeighborCode];
  finalSelf->neighborCodeName = 
    [lattice getNeighborCodeName: finalSelf->neighborCode];

  probeMap = [EmptyProbeMap createBegin: [finalSelf getZone]];
  [probeMap setProbedClass: [finalSelf class]];
  probeMap = [probeMap createEnd];

  messageProbe = 
    [probeLibrary getProbeForMessage: "nextCode"
      inClass: [finalSelf class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "previousCode"
      inClass: [finalSelf class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "continue"
      inClass: [finalSelf class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "neighborCodeName"
       inClass: [finalSelf class]]];

  [probeLibrary setProbeMap: probeMap For: [finalSelf class]];

  [probeDisplayManager createProbeDisplayFor: finalSelf];

  return (finalSelf);
}

-(NeighborCode) getNeighborCode {
  done = NO;
  while (!done)
    [modelActionCache doTkEvents];

  return (neighborCode);
}

-(void) nextCode {
  neighborCode = [lattice getNextNeighborCode: neighborCode];
  neighborCodeName = [lattice getNeighborCodeName: neighborCode];
}

-(void) previousCode {
  neighborCode = [lattice getPreviousNeighborCode: neighborCode];
  neighborCodeName = [lattice getNeighborCodeName: neighborCode];
}

-(id) continue {
  done = YES;

  return self;
}

@end
