/* 
  WorldManager.m

  Barry McMullin <mcmullin@eeng.dcu.ie>
  17-OCT-1996

*/

#import <stdio.h>

#import <random.h>
#import <simtools.h>

#import "SCLGlobals.h"
#import "SCLDisplay.h"

#import "Version.h"

#import "Bernoulli.h"

#import "Hole.h"
#import "Substrate.h"
#import "Link.h"
#import "Catalyst.h"

#import "CellEditor.h"
#import "UserCellEditor.h"
#import "VNneighborhood.h"
#import "MooreNeighborhood.h"
#import "FHPneighborhood.h"
#import "Lattice2d.h"
#import "ParticleManager.h"

#import "WorldManager.h"


@implementation WorldManager


+createBegin: (id) aZone {
  WorldManager * finalSelf;
  id <MessageProbe> messageProbe;
  id <ProbeMap> probeMap;

  finalSelf = [super createBegin: aZone];

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [finalSelf class]];
  probeMap = [probeMap createEnd];

  messageProbe = 
    [probeLibrary getProbeForMessage: "saveToFileNamed:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "loadFromFileNamed:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "sbrkKByte"
       inClass: self]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "worldTime"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "brkWorldTime"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "repairMotionCount"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "displaceMotionCount"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForVariable: "spliceCount"
       inClass: [finalSelf class]]];

  messageProbe = 
    [probeLibrary getProbeForMessage: "kickPRNG"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "createEmptyVNWorldWithX:Y:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "createEmptyMooreWorldWithX:Y:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "createEmptyFHPWorldWithX:Y:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "createDefaultWorld"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "fillWithHoles"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "fillWithSubstrate"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  [probeMap addProbe: 
    [probeLibrary getProbeForMessage: "adjustSubstrateNumberBy:"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForMessage: "adjustLinkNumberBy:"
       inClass: [finalSelf class]]];
  [probeMap addProbe: 
    [probeLibrary getProbeForMessage: "adjustCatalystNumberBy:"
       inClass: [finalSelf class]]];

  messageProbe = 
    [probeLibrary getProbeForMessage: "setWorldTime:"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  //messageProbe = 
  //  [probeLibrary getProbeForMessage: "refreshDisplay"
  //    inClass: [self class]];
  //[messageProbe setHideResult: 1];
  //[probeMap addProbe: messageProbe];

  messageProbe = 
    [probeLibrary getProbeForMessage: "update"
      inClass: [self class]];
  [messageProbe setHideResult: 1];
  [probeMap addProbe: messageProbe];

  [probeLibrary setProbeMap: probeMap For: [finalSelf class]];
  
  return finalSelf;
}


-createEnd {
  WorldManager *finalSelf;

  finalSelf = [super createEnd];
  finalSelf->probeDisplay =
    [probeDisplayManager createProbeDisplayFor: finalSelf];

  finalSelf->worldTime = 0U;
  finalSelf->repairMotionCount = 0U;
  finalSelf->displaceMotionCount = 0U;
  finalSelf->spliceCount = 0U;
  [Link setupProbeMap: [finalSelf getZone]];  // Has to be done *somewhere*...

  return (finalSelf);
}

-(void) drop {
  [self dropWorld];
  [probeDisplay drop];
  [super drop];
}

// Would prefer this to return void, but probe's can't
// seem to handle that (v 1.1)
-(id) kickPRNG {
  if (prng != nil)
    [prng setStateFromSeed: RANDOMSEED];
  else
    [WarningMessage raiseEvent: "kickPRNG: prng is nil!"];

  return self;
}

-(unsigned) setWorldTime: (unsigned) time {
  worldTime = time;

  return worldTime;
}

-(unsigned) getWorldTime {
  return worldTime;
}

-(void) setRepairMotionCount: (unsigned) inCount {
  repairMotionCount = inCount;
}

-(void) setDisplaceMotionCount: (unsigned) inCount {
  displaceMotionCount = inCount;
}

-(void) setSpliceCount: (unsigned) inCount {
  spliceCount = inCount;
}


-refreshDisplay{
  if (world != nil) {
    [worldDisplay displayLattice];
    [probeDisplayManager update];
  }

  return self;
}

-(void) logRepairMotion {
  repairMotionCount++;
}

-(void) logDisplaceMotion {
  displaceMotionCount++;
}

-(void) logSplice {
  spliceCount++;
}

-update {
  sbrkKByte = (int)(sbrk(0))/1024;
  if (world != nil) { // Defensive...
    [holeMgr processQueues];
    [substrateMgr processQueues];
    [linkMgr processQueues];
    [catalystMgr processQueues];

    [bondMgr processQueues];

    [userCellEditor update];

    [self refreshDisplay];
  }

  return self;
}

-step {
  sbrkKByte = (int)(sbrk(0))/1024;
  if (world != nil) { // Defensive...
    worldTime++;

    [[catalystMgr getAgentList] forEach: M(initialiseStep)];
    [[linkMgr getAgentList] forEach: M(initialiseStep)];
    [[substrateMgr getAgentList] forEach: M(initialiseStep)];
    [[holeMgr getAgentList] forEach: M(initialiseStep)];

    [[catalystMgr getAgentList] forEach: M(step)];
    [catalystMgr processQueues];
    [holeMgr processQueues];
    [substrateMgr processQueues];
    [linkMgr processQueues];
    [bondMgr processQueues];

    [[linkMgr getAgentList] forEach: M(step)];
    [holeMgr processQueues];
    [substrateMgr processQueues];
    [linkMgr processQueues];

    //    [bondMgr xprintStatus];
    //    fflush(stdout);

    [bondMgr processQueues];

//     [bondMgr xprintStatus];
//     fflush(stdout);

    [[substrateMgr getAgentList] forEach: M(step)];
    [bondMgr processQueues];
    [[holeMgr getAgentList] forEach: M(step)];
    [bondMgr processQueues];
    [[bondMgr getAgentList] forEach: M(step)];
    [bondMgr processQueues];

    [userCellEditor update];

    [self refreshDisplay];
  }

  if ((brkWorldTime > 0) && (worldTime >= brkWorldTime))
    [modelSwarm setStateStopped];
  return self;
}


-dropWorld {
  [modelSwarm setStateStopped]; // Defensive...
  if (world != nil) {
    /* We assume that if world != nil then *all* of the
       associated objects must be dropped... conversely, if
       world == nil already, we assume *none* of the associated
       objects need be dropped. */

    [prng drop];
    prng = nil;
    [bernoulli drop];
    bernoulli = nil;
    [uniformUnsigned drop];
    uniformUnsigned = nil;
    [uniformDouble drop];
    uniformDouble = nil;

    [userCellEditor drop];
    userCellEditor = nil;

    [catalystMgr drop];
    catalystMgr = nil;
    [linkMgr drop];
    linkMgr = nil;
    [substrateMgr drop];
    substrateMgr = nil;
    [holeMgr drop];
    holeMgr = nil;

    [bondMgr drop];
    bondMgr = nil;
    // NB: bondMgr must be dropped *after* linkMgr,
    // because links attempt to clear their bonds in
    // dropping - which won`t work if the bonds have
    // already all been dropped!

    [worldDisplay drop];
    worldDisplay = nil;

    [world drop];

    world = nil;
  }

  return self;
}

-createWorldWithX: (int) xSize Y: (int) ySize {
  [self dropWorld]; // Defensive...

  prng = [PMMLCG1gen create: [self getZone] 
           setStateFromSeed: DEFAULTSEED];

  bernoulli = [Bernoulli create: [self getZone] setGenerator: prng];
  uniformDouble = [UniformDoubleDist create: 
    [self getZone] setGenerator: prng];
  uniformUnsigned = [UniformUnsignedDist create: 
    [self getZone] setGenerator: prng];

  world = [Lattice2d createBegin: [self getZone]];
  [world setSizeX: xSize Y: ySize];
  world = [world createEnd];
  [world setUniformUnsigned: uniformUnsigned];  

  [Particle setBernoulli: bernoulli];
  [Particle setUniformUnsigned: uniformUnsigned];
  [Bond setBernoulli: bernoulli];
  [Catalyst setBernoulli: bernoulli];
  [Link setBernoulli: bernoulli];
  [Link setUniformDouble: uniformDouble];
  [Link setUniformUnsigned: uniformUnsigned];

  catalystMgr = [ParticleManager createBegin: [self getZone]];
  [catalystMgr setAgentClass: [Catalyst class]];
  [catalystMgr setWorld: world];
  catalystMgr = [catalystMgr createEnd];
  [catalystMgr setUniformUnsigned: uniformUnsigned];  

  linkMgr = [ParticleManager createBegin: [self getZone]];
  [linkMgr setAgentClass: [Link class]];
  [linkMgr setWorld: world];
  linkMgr = [linkMgr createEnd];
  [linkMgr setUniformUnsigned: uniformUnsigned];  

  substrateMgr = [ParticleManager createBegin: [self getZone]];
  [substrateMgr setAgentClass: [Substrate class]];
  [substrateMgr setWorld: world];
  substrateMgr = [substrateMgr createEnd];
  [substrateMgr setUniformUnsigned: uniformUnsigned];  

  holeMgr = [ParticleManager createBegin: [self getZone]];
  [holeMgr setAgentClass: [Hole class]];
  [holeMgr setWorld: world];
  holeMgr = [holeMgr createEnd];
  [holeMgr setUniformUnsigned: uniformUnsigned];  

  bondMgr = [AgentManager createBegin: [self getZone]];
  [bondMgr setAgentClass: [Bond class]];
  bondMgr = [bondMgr createEnd];
  [bondMgr setUniformUnsigned: uniformUnsigned];  

  worldDisplay = [SCLDisplay createBegin: [self getZone]];
  [worldDisplay setLatticeToDisplay: world];
  worldDisplay = [worldDisplay createEnd];

  userCellEditor = [UserCellEditor create: [self getZone] withLattice: world];

  return self;
}

-createEmptyWorldWithX: (int) xSize Y: (int) ySize {
  id originCoord, currentCoord;
  Hole *hole;

  [self createWorldWithX: xSize Y: ySize];
  originCoord = [world createOriginCoord: [self getZone]];
  currentCoord = [world createOriginCoord: [self getZone]];
  do {
    hole = [holeMgr createAgent];
    if (![hole attemptUnwarpToCoord: currentCoord])
      [InternalError raiseEvent:
	"Can't unwarp hole.\n"];
    [currentCoord traverseLattice];
  } while (![currentCoord isEqualToCoord: originCoord]);

  [originCoord drop];
  [currentCoord drop];

  [self setWorldTime: 0U];
  [self setRepairMotionCount: 0U];
  [self setDisplaceMotionCount: 0U];
  [self setSpliceCount: 0U];

  [self update];

  return self;
}

-createEmptyVNWorldWithX: (int) xSize Y: (int) ySize {
  [self createEmptyWorldWithX: xSize Y: ySize];
  [world setNeighborhoodCode: VN];

  return self;
}

-createEmptyMooreWorldWithX: (int) xSize Y: (int) ySize {
  [self createEmptyWorldWithX: xSize Y: ySize];
  [world setNeighborhoodCode: Moore];

  return self;
}

-createEmptyFHPWorldWithX: (int) xSize Y: (int) ySize {
  [self createEmptyWorldWithX: xSize Y: ySize];
  [world setNeighborhoodCode: FHP];

  return self;
}

-dropAgent: (id) agent {
  Class agentClass;

  agentClass = [agent class];
  if (agentClass != nil) { // Defensive
    if (agentClass == [Catalyst class]) 
      [catalystMgr dropAgent: agent];
    else if (agentClass == [Link class]) 
      [linkMgr dropAgent: agent];
    else if (agentClass == [Substrate class])
      [substrateMgr dropAgent: agent];
    else if (agentClass == [Hole class])
      [holeMgr dropAgent: agent];
    else
      [InternalError raiseEvent:
        "Unrecognised agentClass %d encountered.",
        agentClass];
  }

  return self;
}
    

-fillWithSubstrate {
  id originCoord, currentCoord;
  id agent;
  Substrate *substrate;

  originCoord = [world createOriginCoord: [self getZone]];
  currentCoord = [world createOriginCoord: [self getZone]];
  do {
    agent = [currentCoord getObject];
    if ([agent class] != [Substrate class]) {
      [self dropAgent: agent];
      substrate = [substrateMgr createAgent];
      if (![substrate attemptUnwarpToCoord: currentCoord])
	[InternalError raiseEvent:
	"Can't unwarp substrate.\n"];
    }
    [currentCoord traverseLattice];
  } while (![currentCoord isEqualToCoord: originCoord]);

  [originCoord drop];
  [currentCoord drop];

  [self update];

  return self;
}

-(int) addRandomSubstrate: (int) numNewSubstrates {
  id coord;
  int numHoles;
  int newSubstrateCount;
  Hole *hole;
  Substrate *substrate;

  [self update]; // Defensive...
  newSubstrateCount = 0;
  numHoles = [holeMgr getCount];
  while ((newSubstrateCount < numNewSubstrates) && (numHoles > 0)) {
    hole = [holeMgr randomAgent];
    coord = [hole copyCoord: [self getZone]];
    [holeMgr dropAgent: hole];
    numHoles--;
    substrate = [substrateMgr createAgent];
    if (![substrate attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp substrate.\n"];
    [coord drop];
    newSubstrateCount++;
    [holeMgr processQueues];
    [substrateMgr processQueues];
  }

  [substrateMgr processQueues];

  [self update];

  return newSubstrateCount;
}

-(int) removeRandomSubstrate: (int) numRemovedSubstrates {
  id coord;
  int numSubstrates;
  int removedSubstrateCount;
  Hole *hole;
  Substrate *substrate;

  [self update]; // Defensive...
  removedSubstrateCount = 0;
  numSubstrates = [substrateMgr getCount];
  while ((removedSubstrateCount < numRemovedSubstrates) 
      && (numSubstrates > 0)) {
    substrate = [substrateMgr randomAgent];
    coord = [substrate copyCoord: [self getZone]];
    [substrateMgr dropAgent: substrate];
    hole = [holeMgr createAgent];
    if (![hole attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp hole.\n"];
    numSubstrates--;
    removedSubstrateCount++;
    [substrateMgr processQueues];
    [holeMgr processQueues];
  }

  [substrateMgr processQueues];

  [self update];

  return removedSubstrateCount;
}

-(int) adjustSubstrateNumberBy: (int) adjustment {
  if (adjustment > 0)
    adjustment = [self addRandomSubstrate: adjustment];
  else
    if (adjustment < 0)
      adjustment = -([self removeRandomSubstrate: (-adjustment)]);

  return adjustment;
}


-(int) addRandomLink: (int) numNewLinks {
  id coord;
  int numHoles;
  int newLinkCount;
  Hole *hole;
  Link *link;

  [self update]; // Defensive...
  newLinkCount = 0;
  numHoles = [holeMgr getCount];
  while ((newLinkCount < numNewLinks) && (numHoles > 0)) {
    hole = [holeMgr randomAgent];
    coord = [hole copyCoord: [self getZone]];
    [holeMgr dropAgent: hole];
    numHoles--;
    link = [linkMgr createAgent];
    if (![link attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp link).\n"];
    [coord drop];
    newLinkCount++;
    [holeMgr processQueues];
    [linkMgr processQueues];
  }

  [linkMgr processQueues];

  [self update];

  return newLinkCount;
}

-(int) removeRandomLink: (int) numRemovedLinks {
  id coord;
  int numLinks;
  int removedLinkCount;
  Hole *hole;
  Link *link;

  [self update]; // Defensive...
  removedLinkCount = 0;
  numLinks = [linkMgr getCount];
  while ((removedLinkCount < numRemovedLinks) 
      && (numLinks > 0)) {
    link = [linkMgr randomAgent];
    coord = [link copyCoord: [self getZone]];
    [linkMgr dropAgent: link];
    hole = [holeMgr createAgent];
    if (![hole attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
	"Can't unwarp hole.\n"];
    [coord drop];
    numLinks--;
    removedLinkCount++;
    [linkMgr processQueues];
    [holeMgr processQueues];
  }

  [linkMgr processQueues];

  [self update];

  return removedLinkCount;
}

-(int) adjustLinkNumberBy: (int) adjustment {
  if (adjustment > 0)
    adjustment = [self addRandomLink: adjustment];
  else
    if (adjustment < 0)
      adjustment = -([self removeRandomLink: (-adjustment)]);

  return adjustment;
}

-fillWithHoles {
  id originCoord, currentCoord;
  id agent;
  Hole *hole;

  originCoord = [world createOriginCoord: [self getZone]];
  currentCoord = [world createOriginCoord: [self getZone]];
  do {
    agent = [currentCoord getObject];
    if ([agent class] != [Hole class]) {
      [self dropAgent: agent];
      hole = [holeMgr createAgent];
      if(![hole attemptUnwarpToCoord: currentCoord])
	[InternalError raiseEvent:
	"Can't unwarp hole.\n"];
    }
    [currentCoord traverseLattice];
  } while (![currentCoord isEqualToCoord: originCoord]);

  [originCoord drop];
  [currentCoord drop];

  [self update];

  return self;
}


-(int) addRandomCatalyst: (int) numNewCatalysts {
  id coord;
  int numHoles;
  int newCatalystCount;
  Hole *hole;
  Catalyst *catalyst;

  [self update]; // Defensive...
  newCatalystCount = 0;
  numHoles = [holeMgr getCount];
  while ((newCatalystCount < numNewCatalysts) && (numHoles > 0)) {
    hole = [holeMgr randomAgent];
    coord = [hole copyCoord: [self getZone]];
    [holeMgr dropAgent: hole];
    numHoles--;
    catalyst = [catalystMgr createAgent];
    if (![catalyst attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
      "Can't unwarp catalyst.\n"];
    [coord drop];
    newCatalystCount++;
    [holeMgr processQueues];
    [catalystMgr processQueues];
  }

  [catalystMgr processQueues];

  [self update];

  return newCatalystCount;
}

-(int) removeRandomCatalyst: (int) numRemovedCatalysts {
  id coord;
  int numCatalysts;
  int removedCatalystCount;
  Hole *hole;
  Catalyst *catalyst;

  [self update]; // Defensive...
  removedCatalystCount = 0;
  numCatalysts = [catalystMgr getCount];
  while ((removedCatalystCount < numRemovedCatalysts) 
      && (numCatalysts > 0)) {
    catalyst = [catalystMgr randomAgent];
    coord = [catalyst copyCoord: [self getZone]];
    [catalystMgr dropAgent: catalyst];
    hole = [holeMgr createAgent];
    if (![hole attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
      "Can't unwarp hole.\n"];
    [coord drop];
    numCatalysts--;
    removedCatalystCount++;
    [catalystMgr processQueues];
    [holeMgr processQueues];
  }

  [catalystMgr processQueues];

  [self update];

  return removedCatalystCount;
}

-(int) adjustCatalystNumberBy: (int) adjustment {
  if (adjustment > 0)
    adjustment = [self addRandomCatalyst: adjustment];
  else
    if (adjustment < 0)
      adjustment = -([self removeRandomCatalyst: (-adjustment)]);

  return adjustment;
}


-createDefaultWorld {
  id coord;
  int i;
  Particle *particle;
  Substrate *substrate;
  Catalyst *catalyst;
  BOOL holeFound;

  int initialNumSubstrates = 200;
  int initialNumCatalysts = 5;
  int xSize = 20;
  int ySize = 20;

  [self createEmptyWorldWithX: xSize Y: ySize];

  // Add the substrates...
  i = 0;
  while (i < initialNumSubstrates) {
    // Find a hole...
    holeFound = NO;
    while(!holeFound) {
      // This is going to get very slooooow if the 
      // material particle density in the world gets anywhere
      // near 1.0!!!
      coord = [world createRandomCoord: [self getZone]];
      particle = [coord getObject];
      if ([particle isMemberOf: [Hole class]])
        holeFound = YES;
      else
        [coord drop];
    }

    [holeMgr dropAgent: particle];
    substrate = [substrateMgr createAgent];
    if (![substrate attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
      "Can't unwarp hole.\n"];
    [coord drop];

    i++;
  }
  [holeMgr processQueues];
  [substrateMgr processQueues];

  // Add the catalysts...
  i = 0;
  while (i < initialNumCatalysts) {
    // Find a hole...
    holeFound = NO;
    while(!holeFound) {
      // This is going to get very slooooow if the 
      // material particle density in the world gets anywhere
      // near 1.0!!!
      coord = [world createRandomCoord: [self getZone]];
      particle = [coord getObject];
      if ([particle isMemberOf: [Hole class]])
        holeFound = YES;
      else
	[coord drop];
    }

    [holeMgr dropAgent: particle];
    catalyst = [catalystMgr createAgent];
    if (![catalyst attemptUnwarpToCoord: coord])
      [InternalError raiseEvent:
      "Can't unwarp catalyst.\n"];
    [coord drop];

    i++;
  }
  [holeMgr processQueues];
  [catalystMgr processQueues];

  [userCellEditor update];
  [self refreshDisplay];

  return self;
}

-(void) saveInternalState: (id) anObject To: (id) outFile {
  int stateSize, i;
  void *state, *statePtr;

  [outFile putString: "# Internal State: "];
  [outFile putString: [anObject name]];
  [outFile putNewLine];

  stateSize = [anObject getStateSize];
  [outFile putInt: stateSize];
  [outFile putNewLine];
  state = [[self getZone] alloc: stateSize];
  [anObject putStateInto: state];

  statePtr = state;
  for (i = 0; i < stateSize; i++) {
    [outFile putString: " "];
    [outFile putInt: (int)(*(char *)(statePtr++))];
  }

  [[self getZone] free: state];
  [outFile putNewLine];
}

-saveTo: (id <OutFile>) outFile {
  [outFile putString: "# World state...\n"];

  [outFile putString: "# World Time...\n"];
  [outFile putInt: [worldManager getWorldTime]];  // Really need putUnsigned!
  [outFile putNewLine];

  [outFile putString: "# repairMotionCount...\n"];
  [outFile putInt: repairMotionCount];
  [outFile putNewLine];

  [outFile putString: "# displaceMotionCount...\n"];
  [outFile putInt: displaceMotionCount];
  [outFile putNewLine];

  [outFile putString: "# spliceCount...\n"];
  [outFile putInt: spliceCount];
  [outFile putNewLine];


  [outFile putString: "# World Size...\n"];
  [outFile putInt: [world getSizeX]];
  [outFile putString: " "];
  [outFile putInt: [world getSizeY]];
  [outFile putNewLine];

  [outFile putString: "# World NeighborhoodCode...\n"];
  [outFile putInt: [world getNeighborhoodCode]];
  [outFile putNewLine];

  [self saveInternalState: prng To: outFile];
  [self saveInternalState: bernoulli To: outFile];
  [self saveInternalState: uniformDouble To: outFile];
  [self saveInternalState: uniformUnsigned To: outFile];

  [holeMgr saveTo: outFile];
  [substrateMgr saveTo: outFile];
  [linkMgr saveTo: outFile];
  [catalystMgr saveTo: outFile];

  [bondMgr saveTo: outFile];

  return self;
}

-(BOOL) padWithHoles {
  id originCoord, currentCoord;
  id currentAgent;
  Hole *hole;
  BOOL padded;

  padded = NO;
  originCoord = [world createOriginCoord: [self getZone]];
  currentCoord = [world createOriginCoord: [self getZone]];
  do {
    currentAgent = [currentCoord getObject];
    if (currentAgent == nil) {
      padded = YES;
      hole = [holeMgr createAgent];
      if (![hole attemptUnwarpToCoord: currentCoord])
	[InternalError raiseEvent:
	  "Can't unwarp hole.\n"];
    }
    [currentCoord traverseLattice];
  } while (![currentCoord isEqualToCoord: originCoord]);

  [originCoord drop];
  [currentCoord drop];

  return (padded);
}

-(void) loadInternalState: (id) anObject From: (id) inFile {
  int stateSize, i, datum;
  void *state, *statePtr;

  [inFile skipLine]; // "# Internal State: (anObject)"];

  [inFile getInt: &stateSize];
  [inFile skipLine];

  state = [[self getZone] alloc: stateSize];
  statePtr = state;
  for (i = 0; i < stateSize; i++) {
    [inFile getInt: &datum];
    *(char *)(statePtr++) = (char)(datum);
  }

  [anObject setStateFrom: state];
  // Note that this can/will bomb if we try to ship
  // saved files between different platforms, due to
  // incompatibility of internal representations
  // (specificially endian-ness...). Darn.

  [[self getZone] free: state];
  [inFile skipLine];
}


-loadFrom: (id <InFile>) inFile {
  int intParm;
  int xSize, ySize;
  // char c;

  [self dropWorld];

  [inFile skipLine]; // "# World state...\n" 

  //   [inFile getChar: &c];
  // while (c != '\n') {
  //   putchar(c);
  //   [inFile getChar: &c];
  //  }
  // fflush(stdout);

  [inFile skipLine]; // "# World Time...\n"
  [inFile getInt: &intParm]; // Really need getUnsigned!
  [self setWorldTime: intParm];
  [inFile skipLine];

  [inFile skipLine]; // "# repairMotionCount...\n"
  [inFile getInt: &intParm]; // Really need getUnsigned!
  [self setRepairMotionCount: intParm];
  [inFile skipLine];

  [inFile skipLine]; // "# displaceMotionCount...\n"
  [inFile getInt: &intParm]; // Really need getUnsigned!
  [self setDisplaceMotionCount: intParm];
  [inFile skipLine];

  [inFile skipLine]; // "# spliceCount...\n"
  [inFile getInt: &intParm]; // Really need getUnsigned!
  [self setSpliceCount: intParm];
  [inFile skipLine];

  [inFile skipLine]; // "# World Size...\n"
  [inFile getInt: &xSize];
  [inFile getInt: &ySize];
  [inFile skipLine];
  
  // OK, now we have the wherewithall to create the "empty"
  // configuration...
  [self createWorldWithX: xSize Y: ySize];

  [inFile skipLine]; // "# World NeighborhoodCode...\n"
  [inFile getInt: &intParm];
  switch (intParm) {
  case VN:
  case Moore:
  case FHP:
    [world setNeighborhoodCode: intParm]; 
    break;
  default:
    [InternalError raiseEvent:
      "Invalid NeighborhoodCode on loading world.\n"];
  }

  [inFile skipLine];

  [self loadInternalState: prng From: inFile];
  [self loadInternalState: bernoulli From: inFile];
  [self loadInternalState: uniformDouble From: inFile];
  [self loadInternalState: uniformUnsigned From: inFile];

  [holeMgr loadFrom: inFile toLattice: world];
  [substrateMgr loadFrom: inFile toLattice: world];
  [linkMgr loadFrom: inFile toLattice: world];
  [catalystMgr loadFrom: inFile toLattice: world];

  [bondMgr loadFrom: inFile toLattice: world];

  if ([self padWithHoles]) // Defensive...
    [WarningMessage raiseEvent:
       "Saved world state incomplete; padded with extra holes.\n"];

  [self update];
 
  return self;
}

-saveToFileNamed: (char *) filename {
  id <OutFile> outFile;

  [modelSwarm setStateStopped]; // Defensive...

  outFile = [OutFile create: [self getZone] setName: filename];

  if (outFile == nil)
    [WarningMessage raiseEvent: 
    "worldManager: saveToFileNamed: outFile is nil!\n"];
  else {
    [outFile putString: "# "];
    [outFile putString: [Version getAppIDString]];
    [outFile putNewLine];

    [self saveTo: outFile];

    [outFile drop];
  }

  return self;
}

-loadFromFileNamed: (char *) filename {
  id <InFile> inFile;

  [modelSwarm setStateStopped]; // Defensive...

  inFile = [InFile create: [self getZone] setName: filename];

  if (inFile == nil)
    [WarningMessage raiseEvent: 
    "worldManager: loadFromFileNamed: inFile is nil!\n"];
  else {
    [inFile skipLine]; // Version info...
      // Possibly should check for version compatibility (;-)
    [self loadFrom: inFile];
    [inFile drop];
  }

  return self;
}

@end
